/*
 * (c) Copyright 2011, IBM Corporation.
 *
 *      Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef AMINO_UTIL_PPC_H
#define AMINO_UTIL_PPC_H

#ifdef XLC
#include <builtins.h>
#endif

#include <stdint.h>

const int CACHE_PER_CORE=512000;
const int CACHE_LINE_SIZE=64;

//optimization barrier
#define compiler_barrier()  __asm__ __volatile__ ("":::"memory")

//fence between load and load, store and store, load and store, store and load
#define LLFENCE __asm__ __volatile__ ("isync":::"memory")
#define SSFENCE __asm__ __volatile__ ("isync":::"memory")
#define LSFENCE __asm__ __volatile__ ("isync":::"memory")
#define SLFENCE __asm__ __volatile__ ("sync":::"memory")

#define CC_FENCE __asm__ __volatile__ ("sync":::"memory")
#define SC_FENCE __asm__ __volatile__ ("sync":::"memory")


inline int getProcessNum(){
    //TODO add code to get this information
    return 8;
}


/**
 * @brief This function is used to do CAS to two consecutive machine words.
 *
 * Since normal cas() can at most apply to one machine word which has as many
 * bits as address line. This function aims to provide CAS operation on double
 * width data. Eg, for 32bit platform, this function can operate 64bit data;
 * for 64bit platform, this function can operate 128bit data.
 *
 * @param ptr The memory address to be updated atomicially
 * @param old_addr A pointer to the old value. Only if the value referenced by
 * address equals to the old value, CompareAndSet operation can succeed.
 * @param new_v The new value which will be put to address when CAS succeeds
 *
 * @return true, if a compareAndSet operation succeeds; else return false. 
 */
inline bool casd(volatile void *ptr, void *old_addr, const void * new_v) {
    unsigned long * old_addr_long = (unsigned long *) old_addr;
    const unsigned long * new_v_long = (const unsigned long *) new_v;
    bool ret=false;
#if defined(BIT64)
#elif defined(BIT32)
#else
#error Unsupported platform
#endif //BIT64

    return ret;
}

/**
 * @brief atomically exchange content of two variables
 *
 * For "swap" operation. xchg always implies a full memory barrier on X86, even
 * without LOCK prefix. It essentially stores the value v into ptr, and return
 * the value in ptr before the store operation. I am not quite sure return
 * "unsigned long" will meet the demand, also for qualifiers of "v"
 */
inline unsigned long xchg(volatile void * ptr, unsigned long v, int size)
{
    // TODO: Now this is unsupported on AIX platform yet
    return 0;
}
#endif
